#include <mex.h>
#include <matrix.h>
#include <mat.h>

#define MEX
#include "readMps.h"

void
mexFunction(int nlhs, mxArray *plhs[],
            int nrhs, const mxArray *prhs[])
{
  bool fSilent = false;

  if (nrhs == 2) {
    if (!(mxIsLogical(prhs[1]) || (mxGetNumberOfElements(prhs[1]) != 1))) {
      mexErrMsgTxt("\tSecond input argument (SILENT) must be logical scalar\n");
    }

    fSilent = mxGetLogicals(prhs[1])[0];
  }

  if (!fSilent) {
    mexPrintf("\to========================================o\n"
	      "\t|         MPS Parser for MATLAB          |\n"
	      "\t|       ,-----------------------'        |\n"
	      "\t|                                        |\n"
	      "\t| %38s |\n"
	      "\t| %38s |\n"
	      "\t`========================================'\n\n",
	      copyright, revision);
  }

  char *usage = "\tusage:\n"
    		"\t  lp = readmps(<path/to/mpsfile>)\n\n"
		"\tsee help readmps\n";

  /* check number of right hand side arguments */
  if ((nrhs < 1) || (nrhs > 2)) {
    mexPrintf("%s", usage);
    mexErrMsgTxt("Wrong number of input arguments");
  }

  /* check if first right hand side argument can be a path */
  if (!mxIsChar(prhs[0])) {
    mexPrintf("%s", usage);
    mexErrMsgTxt("First input argument (<path/to/mpsfile>) must be a string");
  }
  
  /* check number of left hand arguments */
  if (nlhs > 1) {
    mexPrintf("%s", usage);
    mexErrMsgTxt("Need one output argument");
  }

  /* parse mps file */
  CLpDouble lp;
  unsigned int buflen = mxGetM(prhs[0]) * mxGetN(prhs[0]) * sizeof(mxChar)+ 1;
  char *path = (char *) MALLOC(buflen);
  char *matPath = NULL;
  mxGetString(prhs[0], path, buflen);

  if (readMps(path, lp, fSilent) == -1) {
    mexErrMsgTxt("Could not parse MPS file\n");
  }
  mxFree(path);

  buildABc(lp);

  /* construct output argument */
  const char *fields[] = {"c", "A", "a", "B", "b", "xl", "xu"};
  plhs[0] = mxCreateStructMatrix(1, 1, 7, fields);

  mxArray *mT;
  /* lp.c */
  mT = mxCreateDoubleMatrix(lp.n, 1, mxREAL);
  mxSetPr(mT, lp.c);
  mxSetFieldByNumber(plhs[0], 0, 0, mT);
  /* lp.A */
  mT = mxCreateSparse(lp.m, lp.n, lp.A.nz, mxREAL);
  mxSetPr(mT, lp.A.pr);
  mxSetIr(mT, lp.A.ir);
  mxSetJc(mT, lp.A.jc);
  mxSetFieldByNumber(plhs[0], 0, 1, mT);
  /* lp.a */
  mT = mxCreateDoubleMatrix(lp.m, 1, mxREAL);
  mxSetPr(mT, lp.a);
  mxSetFieldByNumber(plhs[0], 0, 2, mT);
  /* lp.B */
  mT = mxCreateSparse(lp.p, lp.n, lp.B.nz, mxREAL);
  mxSetPr(mT, lp.B.pr);
  mxSetIr(mT, lp.B.ir);
  mxSetJc(mT, lp.B.jc);
  mxSetFieldByNumber(plhs[0], 0, 3, mT);
  /* lp.b */
  mT = mxCreateDoubleMatrix(lp.p, 1, mxREAL);
  mxSetPr(mT, lp.b);
  mxSetFieldByNumber(plhs[0], 0, 4, mT);
  /* lp.xl */
  mT = mxCreateDoubleMatrix(lp.n, 1, mxREAL);
  mxSetPr(mT, lp.xl);
  mxSetFieldByNumber(plhs[0], 0, 5, mT);
  /* lp.xu */
  mT = mxCreateDoubleMatrix(lp.n, 1, mxREAL);
  mxSetPr(mT, lp.xu);
  mxSetFieldByNumber(plhs[0], 0, 6, mT);
}
